/**
 ** rednavExecCodels.cc
 **
 ** Codels called by execution task rednavExec
 **
 ** Author: Redouane BOUMGHAR
 ** Date: 2010-08-31
 **
 **/

#include <portLib.h>
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <string>
#include <iostream>

#include "server/rednavHeader.h"
#include "rednav/navigator.hpp"

using namespace jafar::lgl;
using namespace jafar::rednav;
using namespace std;
typedef NavPath::const_iterator path_iterator;

// Region Map poster ID that is initialized with its PosterFind request
static POSTER_ID regionMapPosterID;
static POSTER_ID pomPosterId;
//static rmapMap  regionMapData;
static rmapLightMap  regionMapData;
// Exported poster
static GENPOS_TRAJ_POINTS* Path;
static GENPOS_TRAJ_POINTS* CoopPath;
static GENPOS_TRAJ_POINTS* PathRaw;
static GENPOS_TRAJ_POINTS* CoopPathRaw;
// Used objects
static Navigator* lycos = NULL;
// Map coordinates and resolution
static double xRMZero = 0.0;
static double yRMZero = 0.0;
static double cellScale = 1.0;
// Path filters enablers/disablers
static bool applyRDVFilter = true;
static bool applyCommonFilter = true;
// Initial position for cooperative path finding
static NavLocation coopLocStart(0,0);
static bool firstFindCoopPath = true;
// Disable cropping and constraining filters
static bool filtersAreOk = true;

/*------------------------------------------------------------------------
 *
 * rednavExecTaskStart  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
rednavExecTaskStart(int *report)
{
  return OK;
}

/*------------------------------------------------------------------------
 *
 * rednavExecTaskEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
rednavExecTaskEnd(void)
{
	// Cleaning previous graph and quadtree
	delete lycos;
	return OK;
}

/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 *              S_rednav_CANNOT_CONNECT
 *              S_rednav_CANNOT_READ_POSTER
 *              S_rednav_BAD_PARAMS
 *              S_rednav_FAILED_MAP_CREATION
 */

/* rednavInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavInitMain(rednavInitParams *initParams, int *report)
{ //{{{
    // Getting region map poster ID
    //if (rednavrmapMapPosterFind((char*)(initParams->regionMapPosterName), &regionMapPosterID) != OK)
    if (rednavrmapLightMapPosterFind((char*)(initParams->regionMapPosterName), &regionMapPosterID) != OK)
    {
        std::cerr << "# rednav: could not find Region Map poster : " 
		          << initParams->regionMapPosterName << endl;
		(*report) = S_rednav_CANNOT_CONNECT;
        return ETHER;
    }

	if (posterFind(initParams->pomPosterName, &pomPosterId) == ERROR)
	{
		std::cerr << "# rednav: cannot find pom poster : " 
		          << initParams->pomPosterName << std::endl;
		(*report) = S_rednav_CANNOT_CONNECT;
		return ETHER;
	}

    // Getting region map poster ID
    //if (rednavrmapMapPosterRead( regionMapPosterID, &regionMapData) != OK)
    if (rednavrmapLightMapPosterRead( regionMapPosterID, &regionMapData) != OK)
    {
        std::cerr << "# rednav: could not load Region Map rmapMap poster";
		(*report) = S_rednav_CANNOT_READ_POSTER;
        return ETHER;
    }

	if (regionMapData.xNb == 0 || regionMapData.yNb == 0)
	{
		lycos = NULL;
		std::cerr << "# rednav: nothing to load in the Region Map";
		(*report) = OK;
		return ETHER;
	}

	// Region Map geo zero can be anyway from the raster origin point.
	// The geo zero is placed according to xRMZero and yRMZero offsets
	// from the geo zero itself.
	cellScale = regionMapData.scale;
	if (cellScale == 0.0)
	{
		cellScale = 1.0;
	}
    // Origins are inverted since GeoData and Region have interved axis XY-YX
    xRMZero  = regionMapData.yOrigin;
    yRMZero  = regionMapData.xOrigin;

    double* bufferedData = NULL;
    bufferedData = (double*)calloc((regionMapData.xNb * regionMapData.yNb), sizeof(double));
	if (bufferedData == NULL)
	{
		cerr << "# rednav: empty buffered data, could not create map: size "
		     << regionMapData.xNb << "x" << regionMapData.yNb << endl;
		(*report) = S_rednav_FAILED_MAP_CREATION;
		return ETHER;
	}

    // With Region Map indexes behaviour
	int k = 0;
    //rmapCell* cell;
	double charToDoubleProba = 1.0/255.0;
	int ieme = 0;
    for (int i = 0; i < regionMapData.xNb; i++)
    {
		ieme = i*regionMapData.yNb;
        for (int j = 0; j < regionMapData.yNb; j++)
        {
			/***** // for NON LIGHT REGIONMAP POSTER
            cell = &(regionMapData.cells[i*regionMapData.yNb + j]);
            /////bufferedData[j*regionMapData.xNb + i] = cell->probas[RMAP_P_FLAT];
			// Get the probability that the land is flat (traversable)
            bufferedData[k] = cell->probas[RMAP_P_FLAT];
			k++;
			****/ 
			// Light Poster
            bufferedData[k] = (1.0 - charToDoubleProba * (double)regionMapData.probaObstacle[ieme + j]);
			k++;
			
        }
    }

	// Loading navigator
	delete lycos; // in case a static map's been previously loaded
	lycos = new Navigator(bufferedData , regionMapData.yNb, regionMapData.xNb, 1,
	                      xRMZero, yRMZero, 0.0, cellScale);
	if (lycos == NULL)
	{
		(*report) = S_rednav_FAILED_MAP_CREATION;
		return ETHER;
	}

	// These two next parameters can be direct-lived changed with the request:
	//                 rednavSetPathCropParameters
	// Entropy/Knowledge limit P(T/-/O) = 0.47 /-/ 0.53
	// REDNAV_DEFAULT_MAXPATHENTROPY;//0.69135;
	// Security limit : P(Obstace) > 0.7
	// REDNAV_DEFAULT_MAXPATHPROBA;//0.7;
	lycos->setPathCropers(REDNAV_DEFAULT_MAXPATHENTROPY, REDNAV_DEFAULT_MAXPATHPROBA);
	lycos->setPathToolSubSampler(REDNAV_DEFAULT_PATHSUBSAMPLE);

	//------------- Freeing buffered Data
	if (bufferedData != NULL) { free(bufferedData); }

	//------------- Initializing poster direct writing
	Path = (GENPOS_TRAJ_POINTS*)posterAddr (REDNAV_PATH_POSTER_ID);
	memset (Path, 0, sizeof(GENPOS_TRAJ_POINTS));
	// Starting path references at zero in the GENPOS_TRAJ_POINTS structure.
	Path->numRef = 0;
	// No path has been defined yet 
	Path->nbPts = 0;

	//------------- Initializing poster
	CoopPath = (GENPOS_TRAJ_POINTS*)posterAddr (REDNAV_COOPPATH_POSTER_ID);
	memset (CoopPath, 0, sizeof(GENPOS_TRAJ_POINTS));
	// Starting path references at zero in the GENPOS_TRAJ_POINTS structure.
	CoopPath->numRef = 0;
	// No path has been defined yet 
	CoopPath->nbPts = 0;

	//------------- Initializing poster for raw path
	PathRaw = (GENPOS_TRAJ_POINTS*)posterAddr (REDNAV_PATHRAW_POSTER_ID);
	memset (PathRaw, 0, sizeof(GENPOS_TRAJ_POINTS));
	PathRaw->numRef = 0;
	PathRaw->nbPts = 0;

	//------------- Initializing poster for raw cooperative path
	CoopPathRaw = (GENPOS_TRAJ_POINTS*)posterAddr (REDNAV_COOPPATHRAW_POSTER_ID);
	memset (CoopPathRaw, 0, sizeof(GENPOS_TRAJ_POINTS));
	CoopPathRaw->numRef = 0;
	CoopPathRaw->nbPts = 0;


	(*report) = OK;
    return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * UpdateData
 *
 * Description:  Updates the map, the decomposition and the navigation graph
 *
 * Reports:      OK
 *              S_rednav_CANNOT_READ_POSTER
 *              S_rednav_FAILED_MAP_CREATION
 */

/* rednavUpdateDataMain  -  codel EXEC of UpdateData
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavUpdateDataMain(int *report)
{ //{{{
    // Getting region map poster ID
    //if (rednavrmapMapPosterRead( regionMapPosterID, &regionMapData) != OK)
    if (rednavrmapLightMapPosterRead( regionMapPosterID, &regionMapData) != OK)
    {
        std::cerr << "# rednav: could not load Region Map rmapMap poster";
		(*report) = S_rednav_CANNOT_READ_POSTER;
        return FAIL;
    }

	/* If we can not read the poster then we still have a map on which trajectories
	 * can be calculated. They might be wrong due to the lack of data updates.
	 */

	// Reading the new data.

	double* bufferedData = NULL;
	bufferedData = (double*)calloc((regionMapData.xNb * regionMapData.yNb), sizeof(double));

	// With Region Map indexes behaviour
	int k = 0;
    ///rmapCell* cell;
	double charToDoubleProba = 1.0/255.0;
	int ieme = 0;
	for (int i = 0; i < regionMapData.xNb; i++)
	{
		ieme = i*regionMapData.yNb;
		for (int j = 0; j < regionMapData.yNb; j++)
		{
			/*** // HEAVY POSTER
			cell = &(regionMapData.cells[i*regionMapData.yNb + j]);
            bufferedData[k] = cell->probas[RMAP_P_FLAT];
			k++;
			****/

			// LIGHT POSTER
            bufferedData[k] = (1.0 - charToDoubleProba * (double)regionMapData.probaObstacle[ieme + j]);
			k++;
		}
	}

	if ( !(lycos->updateMap(bufferedData, regionMapData.yNb, regionMapData.xNb)) )
	{
		(*report) = S_rednav_FAILED_MAP_CREATION;
		return ETHER;
	}

	//------------- Freeing buffered Data
	if (bufferedData != NULL) { free(bufferedData); }

	(*report) = OK;
	return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * Save
 *
 * Description: 
 *
 * Reports:      OK
 */

/* rednavSaveMain  -  codel EXEC of Save
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavSaveMain(int *report)
{ //{{{
	cout << "# rednav: saving data in " <<  (char*)(SDI_F->initParams.rednavLogDirectory) << std::endl ;
	time_t T = time(NULL);
	if ( lycos != NULL)
	{
		char geoFile[REDNAV_MAX_LENGTH];
		sprintf(geoFile, "%s/rednav-geodata_%d.tif", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
		char geoFileT[REDNAV_MAX_LENGTH];
		sprintf(geoFileT, "%s/rednav-geotpu_%d", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
		char quadTreeFile[REDNAV_MAX_LENGTH];
		sprintf(quadTreeFile,"%s/rednav-quadtree_%d.gp", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
		lycos->saveData(string(geoFile), string(geoFileT), string(quadTreeFile));
		/*
		char pathFile[REDNAV_MAX_LENGTH];
		sprintf(pathFile, "%s/rednav-path_%d.tif", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
		lycos->savePath(string(pathFile),...);
		*/
	}

	(*report) = OK;
	return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * PrintInfo
 *
 * Description: 
 *
 * Reports:      OK
 */

/* rednavPrintInfo  -  codel EXEC of PrintInfo
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavPrintInfo(int *report)
{ //{{{
	if (lycos != NULL)
	{
		lycos->printInfo();
	}
  return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * escapePath
 *
 * Description: Escape from a difficult zone. Filters are simplified by only
 * using a distance factor from the navigator.
 *
 * Reports:     OK
 *    S_rednav_BAD_START
 *        If start can not be read from pom.
 *    S_rednav_BAD_GOAL
 *        If goal is NULL, which may never happen.
 *    S_rednav_CAN_NOT_FIND_PATH
 *        When first point is in too uncertain or too obstructed area.
 */
ACTIVITY_EVENT
rednavEscapePath(goalParams *goal, int *report)
{ //{{{
	NavPath rawPath;
	NavPath foundPath;
	NavLocation locStart(0,0);
	NavLocation locGoal(0,0);
	double x_new = 0.0;
	double y_new = 0.0;
	double cost = 0.0;
	
	// By default everything is fine
	(*report) = OK;

	// Defining the START point (calling POM ?)
	POM_POS pos;
	if (rednavPOM_POSPosterRead(pomPosterId, &pos) == ERROR)
	{
		cerr << "#EEE# rednav: can not read pom poster" << endl;
		(*report) = S_rednav_BAD_START;
		return FAIL;
	}
	locStart.setPos(
		(int)(((pos.mainToOrigin.euler.y - xRMZero)/cellScale)*2.0),
		(int)(((pos.mainToOrigin.euler.x - yRMZero)/cellScale)*2.0) );
		//cout << "# rednav: Translating start coordinates from: " << xRMZero
		//<< ", " << yRMZero <<  " to " << locStart.pos_x << ", " <<
		//locStart.pos_y << endl;
	
	// Defining the GOAL
	if (goal == NULL)
	{
		(*report) = S_rednav_BAD_GOAL;
		return FAIL;
	} else {
		// Translate from regionMap/POM to LGL frame
		locGoal.setPos(
		    (int)(((goal->y - xRMZero)/cellScale)*2.0),
			(int)(((goal->x - yRMZero)/cellScale)*2.0) );
	}
		//cout << "# rednav: Translating goal coordinates from: " << goal->x <<
		//", " << goal->y <<  " to " << locGoal.pos_x << ", " << locGoal.pos_y
		//<< endl;

	//--- Call to calculate the path in "rawPath"
	if (goal->curvLength > 0.0)
	{
		lycos->setPathToolPathMaxDistance(goal->curvLength);
	} else  {
		lycos->setPathToolPathMaxDistance(12.0);
	}
	time_t T = time(NULL);
	char pathFile[256];
	lycos->findPath(locStart, locGoal, rawPath, cost); //////////////
	sprintf(pathFile, "%s/rednav-path-%d-a-raw.dat", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
	lycos->savePath(string(pathFile), rawPath);
	lycos->applyTool(rawPath, foundPath, PATH_DISTANCE_LIMIT);
	sprintf(pathFile, "%s/rednav-path-%d-b-dist.dat", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
	lycos->savePath(string(pathFile), foundPath);


	// Converting to RegionMap (POM) coordinates
	int N = 0;
	x_new = 0.0;
	y_new = 0.0;
	path_iterator path_it;
	cout << "# rednav: Escape Path Finding number " << Path->numRef 
	     << " : offsets " << xRMZero << " <-> " << yRMZero  
		 << " with cellScale = " << cellScale << endl;
	cout << "# GNUPLOT columns" << endl 
	     << "# X-regionMap  Y-regionMap  X-NavGraph  Y-NavGraph  X-Quadtree Y-Quadtree" << endl;

	//------------ Filling the path poster
	for(path_it = foundPath.begin(); 
	    (N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != foundPath.end());
		++path_it)
	{ 
		// Correcting coordinates in the superimposed LGL frame
		x_new = (double)((*path_it).first).pos_x;
		x_new = (x_new/2.0)*cellScale + xRMZero;	
		y_new = (double)((*path_it).first).pos_y; 
		y_new = (y_new/2.0)*cellScale + yRMZero;
		// Inverted coordinates to fit with regionMap/POM coordinates
		Path->points[N].x = y_new;
		Path->points[N].y = x_new;

		// P3D Validation criteria
		Path->points[N].theta = 0.0;

		//cout << "Path Point " << N << "  (" << y_new << ", " << x_new << ")" << endl;
		cout << y_new << "   " << x_new << "   " 
			 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
			 << (((*path_it).first).pos_x)/2 << "   " << -(((*path_it).first).pos_y)/2 << "     "
			 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;

		if ( cost >= 10000.0 && (((*path_it).second).certainty[2]) >= 0.75 )
		{
			(*report) = S_rednav_CAN_NOT_FIND_PATH;
		}
		N++;
	}  

	// Warning
	if (path_it != foundPath.end())
	{
		// This means we have more than GENPOS_MAX_TRAJECTORY_SIZE way points
		cerr << "#WWW# rednav: Path is not complete to not exceed max trajectory size." << endl;
	}

	// Setting other important parameters
	Path->nbPts = N;
	Path->numRef++;

	return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * findPath
 *
 * Description: 
 *
 * Reports:      OK
 *    S_rednav_BAD_START
 *        If start can not be read from pom.
 *    S_rednav_BAD_GOAL
 *        If goal is NULL, which may never happen.
 *    S_rednav_CAN_NOT_FIND_PATH
 *        When first point is in too uncertain or too obstructed area.
 */

/* rednavFindPath  -  codel EXEC of findPath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavFindPath(goalParams *goal, int *report)
{ //{{{
	NavPath rawPath;
	NavPath foundPath;
	NavLocation locStart(0,0);
	NavLocation locGoal(0,0);
	double x_new = 0.0;
	double y_new = 0.0;
	int N = 0;
	path_iterator path_it;
	
	// Defining the START point (calling POM ?)
	POM_POS pos;
	if (rednavPOM_POSPosterRead(pomPosterId, &pos) == ERROR)
	{
		cerr << "#EEE# rednav: can not read pom poster" << endl;
		(*report) = S_rednav_BAD_START;
		return FAIL;
	}
	locStart.setPos(
		(int)(((pos.mainToOrigin.euler.y - xRMZero)/cellScale)*2.0),
		(int)(((pos.mainToOrigin.euler.x - yRMZero)/cellScale)*2.0) );
		//cout << "# rednav: Translating start coordinates from: " << xRMZero
		//<< ", " << yRMZero <<  " to " << locStart.pos_x << ", " <<
		//locStart.pos_y << endl;
	
	// Defining the GOAL
	if (goal == NULL)
	{
		(*report) = S_rednav_BAD_GOAL;
		return FAIL;
	} else {
		// Translate from regionMap/POM to LGL frame
		locGoal.setPos(
		    (int)(((goal->y - xRMZero)/cellScale)*2.0),
			(int)(((goal->x - yRMZero)/cellScale)*2.0) );
	}
		//cout << "# rednav: Translating goal coordinates from: " << goal->x <<
		//", " << goal->y <<  " to " << locGoal.pos_x << ", " << locGoal.pos_y
		//<< endl;

	// Call to calculate the path in "rawPath"
	//lycos->findPath(locStart, locGoal, rawPath);
	// Crop the found Path according to entropy and obstacle proba
	//lycos->applyTool(rawPath, foundPath, PATH_RDV_CROP);
	//lycos->applyTool(rawPath, foundPath, PATH_SAFESAMPLING);
	lycos->setPathToolSubSampler(8.0);
	time_t T = time(NULL);
	char pathFile[256];
	lycos->findPath(locStart, locGoal, rawPath);
	sprintf(pathFile, "%s/rednav-path-raw-%d.dat", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
	lycos->savePath(string(pathFile), rawPath);
	lycos->applyTool(rawPath, foundPath, PATH_RDV_CROP);
	sprintf(pathFile, "%s/rednav-path-rdv-%d.dat", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
	lycos->savePath(string(pathFile), foundPath);
	lycos->applyTool(foundPath, rawPath, PATH_COMMON_FILTER);
	lycos->applyTool(rawPath, foundPath, PATH_SAFESAMPLING);
	sprintf(pathFile, "%s/rednav-path-final-%d.dat", (char*)(SDI_F->initParams.rednavLogDirectory), (int)T);
	lycos->savePath(string(pathFile), foundPath);

	//////////////////////////////////////////////////////////////
	//     Call to calculatei/find A path in "rawPath"          //
	//     lycos->findPath(locStart, locGoal, rawPath);         //
	//////////////////////////////////////////////////////////////
	lycos->findPath(locStart, locGoal, foundPath);
	//------- saving Raw Path
	N = 0;
	x_new = 0.0;
	y_new = 0.0;
	for(path_it = foundPath.begin(); 
		(N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != foundPath.end());
		++path_it)
	{
		// Correcting coordinates in the superimposed LGL frame
		x_new = (double)((*path_it).first).pos_x;
		x_new = (x_new/2.0)*cellScale + xRMZero;
		y_new = (double)((*path_it).first).pos_y;
		y_new = (y_new/2.0)*cellScale + yRMZero;
		// Inverted coordinates to fit with regionMap/POM coordinates
		Path->points[N].x = y_new;
		Path->points[N].y = x_new;

		// P3D Validation criteria
		Path->points[N].theta = 0.0;

		//cout << "Path Point " << N << "  (" << y_new << ", " << x_new << ")" << endl;
		cout << y_new << "   " << x_new << "   " 
			 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
			 << (((*path_it).first).pos_x)/2 << "   " << -(((*path_it).first).pos_y)/2 << "     "
			 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;
		N++;
	}
	PathRaw->nbPts = N;
	PathRaw->numRef++;

	//------- Apply path filters
	// To tell which object to use and avoid wasting time copying it
	bool useRawPathContainer = false;
	// Crop the found Path according to entropy and obstacle proba
	if (applyRDVFilter)
	{
		lycos->applyTool(foundPath, rawPath, PATH_RDV_CROP);
		useRawPathContainer = true;
	} 
	// Subsample the found Path according to a given interval 
	if (applyCommonFilter && applyRDVFilter)
	{
		lycos->applyTool(rawPath, foundPath, PATH_COMMON_FILTER);
		useRawPathContainer = false;
	} else {
		if (applyCommonFilter)
		{
			lycos->applyTool(foundPath, rawPath, PATH_COMMON_FILTER);
			useRawPathContainer = true;
		}
	}

	//-------- Fill the main path output poster
	cout << "#########################################################################" << endl;
	cout << "# [rednav-genom] Path Finding : number " << Path->numRef 
	     << " : offsets " << xRMZero << " <-> " << yRMZero  
		 << " with cellScale = " << cellScale << endl;
	cout << "# [rednav-genom] Start " << locStart << endl; 
	cout << "# [rednav-genom] Goal  " << locGoal << endl; 
	cout << "#########################################################################" << endl
	     << "# [rednav-genom] GNUPLOT columns for navigation path:" << endl 
	     << "# X-regionMap  Y-regionMap  X-NavGraph  Y-NavGraph  X-Quadtree Y-Quadtree" << endl;
	N = 0;
	x_new = 0.0;
	y_new = 0.0;
	if(useRawPathContainer)
	{
		for(path_it = rawPath.begin(); 
			(N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != rawPath.end());
			++path_it)
		{ 
			// Correcting coordinates in the superimposed LGL frame
			x_new = (double)((*path_it).first).pos_x;
			x_new = (x_new/2.0)*cellScale + xRMZero;
			y_new = (double)((*path_it).first).pos_y; 
			y_new = (y_new/2.0)*cellScale + yRMZero;
			// Inverted coordinates to fit with regionMap/POM coordinates
			Path->points[N].x = y_new;
			Path->points[N].y = x_new;
			Path->points[N].theta = 0.0;
			//cout << "Path Point " << N << "  (" << y_new << ", " << x_new << ")" << endl;
			cout << y_new << "   " << x_new << "   " 
				 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
				 << (((*path_it).first).pos_x)/2 << "   " << -(((*path_it).first).pos_y)/2 << "     "
				 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;
			N++;
		}  
		cout << "#########################################################################" << endl;

		// Warning
		if (path_it != rawPath.end())
		{
			// This means we have more than GENPOS_MAX_TRAJECTORY_SIZE way points
			cerr << "#WWW# rednav: Path is not complete to not exceed max trajectory size." << endl;
		}
	} else { //------------------------------------// use foundPath container
		for(path_it = foundPath.begin(); 
			(N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != foundPath.end());
			++path_it)
		{ 
			// Correcting coordinates in the superimposed LGL frame
			x_new = (double)((*path_it).first).pos_x;
			x_new = (x_new/2.0)*cellScale + xRMZero;	
			y_new = (double)((*path_it).first).pos_y; 
			y_new = (y_new/2.0)*cellScale + yRMZero;
			// Inverted coordinates to fit with regionMap/POM coordinates
			Path->points[N].x = y_new;
			Path->points[N].y = x_new;
			Path->points[N].theta = 0.0;
			//cout << "Path Point " << N << "  (" << y_new << ", " << x_new << ")" << endl;
			cout << y_new << "   " << x_new << "   " 
				 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
				 << (((*path_it).first).pos_x)/2 << "   " << -(((*path_it).first).pos_y)/2 << "     "
				 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;
			N++;
		}  
		cout << "#########################################################################" << endl;

		// Warning
		if (path_it != foundPath.end())
		{
			// This means we have more than GENPOS_MAX_TRAJECTORY_SIZE way points
			cerr << "#WWW# rednav: Path is not complete to not exceed max trajectory size." << endl;
		}
	}

	// Adding the goal to the list
	/*
	if ( N > 0 && (Path->points[N-1].x != y_new) || (Path->points[N-1].y != x_new))
	{
		Path->points[N].x = y_new;
		Path->points[N].y = x_new;
		Path->points[N].theta = 0.0;
		cout << y_new << "   " << x_new << "   " 
			 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
			 << (((*path_it).first).pos_x)/2 << "   " << (((*path_it).first).pos_y)/2 << "     "
			 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;
		N++;
	}
	*/


	// Setting other important parameters
	Path->nbPts = N;
	Path->numRef++;

	if (N == 0)
	{
		(*report) = S_rednav_CAN_NOT_FIND_PATH;
	} else {
		(*report) = OK;
	}
	return ETHER;
} //}}}


/*------------------------------------------------------------------------
 * FindCoopPath
 *
 * Description: Find a cooperative path using utility
 *
 * Reports:      OK
 *              S_rednav_BAD_START
 *              S_rednav_BAD_GOAL
 *              S_rednav_CAN_NOT_FIND_PATH
 *              S_rednav_CAN_NOT_UPDATE_POSTER
 */

/* rednavFindCoopPath  -  codel EXEC of FindCoopPath
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavFindCoopPath(coopParams *coopInfo, int *report)
{ //{{{
	//----------- Creating start and goal points
	NavLocation locGoal(0,0);
	double x_new = 0.0;
	double y_new = 0.0;
	int N = 0;
	// CoopPath iterator
	path_iterator path_it;

	if ( firstFindCoopPath )
	{
		coopLocStart.setPos(
			(int)(((coopInfo->ystart - xRMZero)/cellScale)*2.0),
			(int)(((coopInfo->xstart - yRMZero)/cellScale)*2.0) );
		cout << "# rednavFindCoopPath: Translating Start coordinates from: " 
			 << coopInfo->xstart << ", " << coopInfo->ystart <<  " to " 
			 << coopLocStart.pos_x << ", " << coopLocStart.pos_y << endl;
		firstFindCoopPath = false;
	} // else it is set at the end of this function to the final goal coop point

	locGoal.setPos(
		(int)(((coopInfo->ygoal - xRMZero)/cellScale)*2.0),
		(int)(((coopInfo->xgoal - yRMZero)/cellScale)*2.0) );
	cout << "# rednavFindCoopPath: Translating Goal coordinates from: " 
	     << coopInfo->xgoal << ", " << coopInfo->ygoal <<  " to " 
		 << locGoal.pos_x << ", " << locGoal.pos_y << endl;


	//------------ Finding a utile path
	NavPath rawPath;
	NavPath foundPath;
	lycos->findPathOnUtility(coopLocStart, locGoal, rawPath);
	//------- saving Raw CoopPath
	N = 0;
	x_new = 0.0;
	y_new = 0.0;
	for(path_it = rawPath.begin(); 
		(N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != rawPath.end());
		++path_it)
	{
		// Correcting coordinates in the superimposed LGL frame
		x_new = (double)((*path_it).first).pos_x;
		x_new = (x_new/2.0)*cellScale + xRMZero;
		y_new = (double)((*path_it).first).pos_y;
		y_new = (y_new/2.0)*cellScale + yRMZero;
		// Inverted coordinates to fit with regionMap/POM coordinates
		CoopPathRaw->points[N].x = y_new;
		CoopPathRaw->points[N].y = x_new;
		CoopPathRaw->points[N].theta = 0.0;
		N++;
	}
	CoopPathRaw->nbPts = N;
	CoopPathRaw->numRef++;


	//----------- Apply two points filter
	lycos->applyTool(rawPath, foundPath, PATH_TWO_POINTS_LIMIT);

	//----- Subsampling the path and Filling the cooppath poster
	N = 0;
	for(path_it = foundPath.begin(); 
	    (N < GENPOS_MAX_TRAJECTORY_SIZE) && (path_it != foundPath.end());
		++path_it)
	{ 
		// Correcting coordinates in the superimposed LGL frame
		x_new = (double)((*path_it).first).pos_x;
		x_new = (x_new/2.0)*cellScale + xRMZero;
		y_new = (double)((*path_it).first).pos_y;
		y_new = (y_new/2.0)*cellScale + yRMZero;
		// Inverted coordinates to fit with regionMap/POM coordinates
		CoopPath->points[N].x = y_new;
		CoopPath->points[N].y = x_new;
		CoopPath->points[N].theta = 0.0;
		cout << y_new << "   " << x_new << "   " 
			 << ((*path_it).first).pos_x << "   " << ((*path_it).first).pos_y  << "   "
			 << (((*path_it).first).pos_x)/2 << "   " << -(((*path_it).first).pos_y)/2 << "     "
			 << (((*path_it).second).certainty[0]) << "   " << (((*path_it).second).certainty[1]) << "   " << (((*path_it).second).certainty[2]) << endl;
		N++;
	}  
	// Setting other important parameters
	CoopPath->nbPts = N;
	CoopPath->numRef++;

	// Warning
	if (path_it != foundPath.end())
	{
		// This means we have more than GENPOS_MAX_TRAJECTORY_SIZE way points
		cerr << "# [rednav-genom] warning: Number of waypoints exceeded the maximum trajectory size." << endl;
	}

	// Saving last given position as it will be the next start for the UAV
	coopLocStart.setPos(
		(int)(((x_new - xRMZero)/cellScale)*2.0),
		(int)(((y_new - yRMZero)/cellScale)*2.0)
	);

	// Case where we finished on the goal point
	if (coopLocStart == locGoal)
	{
		// So at next step coopLocStart is set to the given start
		firstFindCoopPath = true;
	}

	(*report) = OK;
	return ETHER;
} //}}}

/*------------------------------------------------------------------------
 * LoadStaticMap
 *
 * Description:  Load an existing file as input map.
 *
 * Reports:      OK
 *              S_rednav_CANNOT_LOAD_MAP
 */

/* rednavLoadStaticMap -  codel EXEC of LoadStaticMap
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavLoadStaticMap(filePath* mapInput, int *report)
{ //{{{
	delete lycos;
	xRMZero = 0.0;
	yRMZero = 0.0;
	cellScale = 1.0;
	lycos = new Navigator(mapInput->mapFile);
	if ( !(lycos->isUp()))
	{
		(*report) = S_rednav_CANNOT_LOAD_MAP;
		return ETHER;
	}

	(*report) = OK;
	return ETHER;
} //}}}



/*------------------------------------------------------------------------
 * SetPathCropParameters
 *
 * Description:  Set path cropping parameters
 *
 * Reports:      OK
 *              S_rednav_WRONG_PARAMS
 */

/* rednavSetPathCropParameters -  codel EXEC of SetPathCropParameters
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavSetPathCropParameters(pathCroper* pathParams, int *report)
{ //{{{
	if (   pathParams->entropyLimit < 0.0
		|| pathParams->probaLimit   < 0.0 )
	{
		(*report) = S_rednav_WRONG_PARAMS;
		return ETHER;
	}

	lycos->setPathCropers(pathParams->entropyLimit, pathParams ->probaLimit);

	(*report) = OK;
	return ETHER;
} //}}}



/*------------------------------------------------------------------------
 * DisableRDVFilter 
 *
 * Description:  Set applyRDVFilter to false
 *
 * Reports:      OK, S_rednav_FILTER_RDV_NOT_DISABLED
 */

/* rednavDisableRDVFilter -  codel EXEC of DisableRDVFilter 
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavDisableRDVFilter(int *report)
{ //{{{
	applyRDVFilter = false;
	(*report) = OK;
	return ETHER;
} //}}}



/*------------------------------------------------------------------------
 * EnableRDVFilter 
 *
 * Description:  Set applyRDVFilter to false
 *
 * Reports:      OK, S_rednav_FILTER_RDV_NOT_ENABLED
 */

/* rednavEnableRDVFilter -  codel EXEC of EnableRDVFilter 
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavEnableRDVFilter(int *report)
{ //{{{
	applyRDVFilter = true;
	(*report) = OK;
	return ETHER;
} //}}}



/*------------------------------------------------------------------------
 * DisableCommonFilter 
 *
 * Description:  Set applyCommonFilter to false
 *
 * Reports:      OK, S_rednav_FILTER_COMMON_NOT_DISABLED
 */

/* rednavDisableCommonFilter -  codel EXEC of DisableCommonFilter 
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavDisableCommonFilter(int *report)
{ //{{{
	applyCommonFilter = false;
	(*report) = OK;
	return ETHER;
} //}}}



/*------------------------------------------------------------------------
 * EnableCommonFilter 
 *
 * Description:  Set applyCommonFilter to false
 *
 * Reports:      OK, S_rednav_FILTER_COMMON_NOT_ENABLED
 */

/* rednavEnableCommonFilter -  codel EXEC of EnableCommonFilter 
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
rednavEnableCommonFilter(int *report)
{ //{{{
	applyCommonFilter = true;
	(*report) = OK;
	return ETHER;
} //}}}


